/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.mapper.rdb.adapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;

/**
 * RdbAdapter for the Hsql Database.
 * 
 * @author S.Chassande-Barrioz
 */
public class HsqlAdapter extends BasicRdbAdapter {

    public HsqlAdapter() {
        super("hsql");
    }

    public boolean supportBatchPreparedStatement() {
        return false;
    }

    public String getIndexedLocateExpression(String instring,
                                             String substring,
                                             String fromIndex) {
        StringBuffer sb = new StringBuffer();
        sb.append("locate(").append(substring).append(", ");
        sb.append(instring).append(", ").append(fromIndex).append(")");
        return sb.toString();
    }

    // overrides existRelation for getTables with table names not supported by
    // driver
    protected boolean existRelation(Connection connection, String tableName,
            String[] relationTypes) throws SQLException {
        boolean existtable = false;
        ResultSet rs = null;

        if (relationTypes == null) {
            relationTypes = new String[] { "TABLE", "VIEW" };
        }

        String schema = null;
        //if a schema is specified: SCHEMA.TABLE
        if (tableName.indexOf('.') != -1) {
        	//SCHEMA
        	schema = tableName.substring(0, tableName.indexOf('.'));
        	//TABLE: without .
        	tableName = tableName.substring(tableName.indexOf('.')+1);
        }
        
        try {
            rs = connection.getMetaData().getTables(null, schema, null,
                    relationTypes);
            while (rs.next() && !existtable) {
                existtable = tableName.equalsIgnoreCase(rs.getString(3));
            }
        } finally {
            if (rs != null) {
                rs.close();
            }
        }
        return existtable;
    }

    public boolean existSequence(Connection connection, String seqName)
            throws SQLException {
        ResultSet rs = null;
        PreparedStatement ps = connection
                .prepareStatement("SELECT * FROM SYSTEM_SEQUENCES WHERE SEQUENCE_NAME=\'"
                        + seqName.toUpperCase() + "\'");
        rs = ps.executeQuery();
        return rs.next();
    }

    public String getNextValInSequence(String seqName) {
        return "SELECT NEXT VALUE FOR " + seqName
                + " FROM SYSTEM_SEQUENCES WHERE SEQUENCE_NAME=\'" + seqName.toUpperCase()
                + "\'";
    }

    public void getManyNextValInSequence(Connection conn, String seqName, int numberOfNextVal) 
    throws RdbAdapterException {
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT ");
		for (int i = 0; i < numberOfNextVal -1; i++) {
			sb.append("NEXT VALUE FOR " + seqName + ", ");
		}
		sb.append("NEXT VALUE FOR " + seqName + 
				" FROM SYSTEM_SEQUENCES WHERE SEQUENCE_NAME=\'" + seqName.toUpperCase()
                + "\'");
		PreparedStatement ps = null;
		try {
			ps = conn.prepareStatement(sb.toString());
			ps.executeQuery();
		} catch (SQLException e) {
			throw new RdbAdapterException(e, "Impossible to allocate " + numberOfNextVal + " identifier"
				+ " on the sequence: " + seqName);
		} finally {
			try {
				if (ps != null) {
					ps.close();
				}
			} catch (SQLException e) {
			}
		}
		
	}
    
    protected void modifySelectClauseWithRange(StringBuffer sb, boolean rangeStart,
            boolean rangeSize) {
        if (rangeSize || rangeStart) {
	        if (rangeStart) {
	            sb.append (" LIMIT ? ");
	        } else {
	            sb.append(" LIMIT 0 ");
	        }
	        if (rangeSize) {
	            sb.append(" ? ");
	        } else {
	            sb.append(" 0 ");
	        }
        }
    }

    protected void modifyQueryWithRange(StringBuffer sb, boolean rangeStart,
            boolean rangeSize) {
        // Do nothing
    }

    public int getRangeParametersAtStart() {
        return RANGEATSTART;
    }
}