/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.mapper.rdb.adapter.api;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.io.Serializable;
import java.io.IOException;

/**
 * Adapter for PreparedStatement management
 *
 * @author S.Chassande-Barrioz
 */
public interface PreparedStatementAdapter {

	/**
	 * Assignes a null value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param sqlType is the type of the column
	 * @throws SQLException
	 */
	void setNull(PreparedStatement ps, int idx, int sqlType)
			throws SQLException;

	/**
	 * Assignes a boolean value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setBoolean(PreparedStatement ps, int idx, boolean o)
			throws SQLException;

	/**
	 * Assignes a Boolean value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setOboolean(PreparedStatement ps, int idx, Boolean o)
			throws SQLException;

	/**
	 * Assignes a char value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setChar(PreparedStatement ps, int idx, char o)
			throws SQLException;

	/**
	 * Assignes a Character value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setOchar(PreparedStatement ps, int idx, Character o)
			throws SQLException;

	/**
	 * Assignes a byte value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setByte(PreparedStatement ps, int idx, byte o)
			throws SQLException;

	/**
	 * Assignes a Byte value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setObyte(PreparedStatement ps, int idx, Byte o)
			throws SQLException;

	/**
	 * Assignes a short value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setShort(PreparedStatement ps, int idx, short o)
			throws SQLException;

	/**
	 * Assignes a Short value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setOshort(PreparedStatement ps, int idx, Short o)
			throws SQLException;

	/**
	 * Assignes an int value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setInt(PreparedStatement ps, int idx, int o)
			throws SQLException;

	/**
	 * Assignes an Integer value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setOint(PreparedStatement ps, int idx, Integer o)
			throws SQLException;

	/**
	 * Assignes a long value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setLong(PreparedStatement ps, int idx, long o)
			throws SQLException;

	/**
	 * Assignes a Long value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setOlong(PreparedStatement ps, int idx, Long o)
			throws SQLException;

	/**
	 * Assignes a float value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setFloat(PreparedStatement ps, int idx, float o)
			throws SQLException;

	/**
	 * Assignes a Float value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setOfloat(PreparedStatement ps, int idx, Float o)
			throws SQLException;

	/**
	 * Assignes a double value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setDouble(PreparedStatement ps, int idx, double o)
			throws SQLException;

	/**
	 * Assignes a Double value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setOdouble(PreparedStatement ps, int idx, Double o)
			throws SQLException;

	/**
	 * Assignes a value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setString(PreparedStatement ps, int idx, String o)
			throws SQLException;

	/**
	 * Assignes a String value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setDate(PreparedStatement ps, int idx, Date o, String columnType)
			throws SQLException;

	/**
	 * Assignes a char[] value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setCharArray(PreparedStatement ps, int idx, char[] o)
			throws SQLException;

	/**
	 * Assignes a byte[] value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setByteArray(PreparedStatement ps, int idx, byte[] o)
			throws SQLException;

	/**
	 * Assignes a BigDecimal value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setBigDecimal(PreparedStatement ps, int idx, BigDecimal o)
			throws SQLException;

	/**
	 * Assignes a BigInteger value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setBigInteger(PreparedStatement ps, int idx, BigInteger o)
			throws SQLException;

	/**
	 * Assignes a serializable value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @throws SQLException
	 */
	void setSerialized(PreparedStatement ps, int idx, Serializable o)
			throws SQLException, IOException;
	
	/**
	 * Assignes an array value into a PreparedStatement
	 * @param ps is the PreparedStatement
	 * @param idx is the index of the the value in the PreparedStatement
	 * @param o is the value to assign
	 * @param typeCode is the type code of the array
	 * ex: = (TypeAdapter.ARRAY + TypeAdapter.STRING)
	 * @throws SQLException
	 */
	void setArray(PreparedStatement ps, int idx, Object o, int typeCode)
			throws SQLException;
	
	String getArrayToString(Object array, int typeCode);	
}
