/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.mapper.rdb.adapter.api;

import java.sql.Connection;
import java.sql.SQLException;

/**
 * Adapter for sequence management.
 *
 * @author S.Chassande-Barrioz
 */
public interface SequenceAdapter {

	/**
	 * @param seqName is the sequence name
	 * @return the SQL command to create a sequence
	 */
	String getCreateSequence(String seqName)
	    throws UnsupportedOperationException ;

	/**
	 *
	 * @param seqName is the sequence name
	 * @param startid is the initial value of the sequence
	 * @param increment is the increment size
	 * @param cache is the size of the cache
	 * @return the SQL command to create a sequence
	 * @throws UnsupportedOperationException
	 */
	String getCreateSequence(String seqName, Integer startid, Integer increment, Integer cache)
	    throws UnsupportedOperationException;

	/**
	 * @param seqName is the sequence name
	 * @return the SQL command to drop a sequence
	 */
	String getDropSequence(String seqName)
		throws UnsupportedOperationException ;
	/**
	 * checks the sequence existence
	 * @param connection is the JDBC connection to use
	 * @param seqName is the sequence of the table
	 * @return true if the table exists, otherwise false.
	 */
	boolean existSequence(Connection connection, String seqName)
			throws SQLException;

	/**
	 * @param seqName is the sequence name
	 * @return the SQL command to gett a new value in a SQL
	 * sequence which the name is given in parameter.
	 */
	String getNextValInSequence(String seqName);
	
	/**
	 * @param seqName the sequence name
	 * @param numberOfNextVal the number of next val called in the same query
	 */
	void getManyNextValInSequence(Connection conn, String seqName, int numberOfNextVal) throws RdbAdapterException;
}
